/* main.c */
#include "global.h"
#include <stdlib.h>
#include <ddraw.h>

#define ID_TIMER            1

/*    */
static const TCHAR szWindowClassName[] = "WINCLASS1";
static const TCHAR szWindowTitle[] = " ";

/*   */
HINSTANCE ghInst;                  /*   */
HWND ghWndMain;                    /*   */

/*   */
static LRESULT CALLBACK WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;
	RECT rect;
	HPEN hpen;
	int i;

	switch (msg) {
		case WM_CREATE:
			SetTimer(hWnd,ID_TIMER,120000,0);
			break;

		case WM_PAINT:
			hdc = BeginPaint(hWnd,&ps);
			EndPaint(hWnd,&ps);
			break;

		case WM_TIMER:
			hdc = GetDC(hWnd);

			hpen = CreatePen(PS_SOLID,1,RGB(0,0,0));
			SelectObject(hdc,hpen);

			GetClientRect(hWnd,&rect);
			for (i=0; i <= rect.bottom; i++) {
				MoveToEx(hdc,0,i,NULL);
				LineTo(hdc,rect.right,i);
				if (i % 2 == 0)
					Sleep(1);
			}

			DeleteObject(hpen);
			ReleaseDC(hWnd,hdc);
			break;

		case WM_DESTROY:
			KillTimer(hWnd,ID_TIMER);
			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd,msg,wParam,lParam);
	}

	return 0;
}

/*     */
static int VersionCheck(void)
{
	OSVERSIONINFO os;

	/*      */
	memset(&os,0,sizeof(OSVERSIONINFO));
	os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&os))
		return 0;

	/*    */
	return (os.dwPlatformId == VER_PLATFORM_WIN32s) ? 0 : 1;
}

/*       */
static int InitializeApp(HINSTANCE hInst, int nCmdShow)
{
	WNDCLASSEX wc;

	/*    */
	ghInst = hInst;

	/*     */
	memset(&wc,0,sizeof(WNDCLASSEX));
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = WindowProc;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(NULL,IDI_APPLICATION);
	wc.hIconSm = LoadIcon(NULL,IDI_APPLICATION);
	wc.hCursor = LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszClassName = szWindowClassName;

	/*    */
	if (!RegisterClassEx(&wc))
		return 0;

	/*   */
	ghWndMain = CreateWindowEx(0,szWindowClassName,szWindowTitle,WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,NULL,NULL,hInst,NULL);
	if (!ghWndMain)
		return 0;

	/*   */
	ShowWindow(ghWndMain,nCmdShow);
	UpdateWindow(ghWndMain);

	return 1;
}

/*     */
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;
	HDC hdc;
	HPEN hpen;
	RECT rect;
	int x1,y1,x2,y2,x1v,y1v,x2v,y2v;
	int mark;
	UCHAR r,g,b;

	/*     */
	if (!VersionCheck()) {
		MessageBox(NULL,"    Win32.",
			"",MB_OK|MB_ICONSTOP);
		return 1;
	}

	/*     */
	if (!InitializeApp(hInst,nCmdShow)) {
		MessageBox(NULL,"    .",
			"",MB_OK|MB_ICONSTOP);
		return 1;
	}

	/*    */
	hdc = GetDC(ghWndMain);
	/*  rand() */
	srand(GetTickCount());

	GetClientRect(ghWndMain,&rect);
	x1 = rand() % rect.right;
	y1 = rand() % rect.bottom;
	x2 = rand() % rect.right;
	y2 = rand() % rect.bottom;

	if ((x1v = rand() % 8 - 4) == 0)
		x1v = 4;
	if ((y1v = rand() % 8 - 4) == 0)
		y1v = 3;
	if ((x2v = rand() % 8 - 4) == 0)
		x2v = 4;
	if ((y2v = rand() % 8 - 4) == 0)
		y2v = 3;

	r = 255; g = 0; b = 0; mark = 1;
	hpen = CreatePen(PS_SOLID,1,RGB(r,g,b));

	/*    */
	for(;;) {
		DWORD dwStart = GetTickCount();
		int timeout;

		/*  ESC */
		if (KEYDOWN(VK_ESCAPE))
			PostMessage(ghWndMain,WM_CLOSE,0,0);

		/*   ,    */
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) {
			/*     */
			if (msg.message == WM_QUIT)
				break;
			/*     */
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			/*    */
			continue;
		}

		/*     */
		x1 += x1v;
		y1 += y1v;
		x2 += x2v;
		y2 += y2v;

		/*     */
		GetClientRect(ghWndMain,&rect);
		if ((x1 < 0) || (x1 > rect.right)) {
			x1v = -x1v;
			/* ,       */
			if (x1 < 0)
				x1 = 0;
			else if (x1 > rect.right)
				x1 = rect.right;
		}
		if ((y1 < 0) || (y1 > rect.bottom)) {
			y1v = -y1v;
			if (y1 < 0)
				y1 = 0;
			else if (y1 > rect.bottom)
				y1 = rect.bottom;
		}

		if ((x2 < 0) || (x2 > rect.right)) {
			x2v = -x2v;
			if (x2 < 0)
				x2 = 0;
			else if (x2 > rect.right)
				x2 = rect.right;
		}
		if ((y2 < 0) || (y2 > rect.bottom)) {
			y2v = -y2v;
			if (y2 < 0)
				y2 = 0;
			else if (y2 > rect.bottom)
				y2 = rect.bottom;
		}

		/*       */
		switch (mark) {
			case 1:
				if (!--r) mark = 2;
				g++;
				break;

			case 2:
				if (!--g) mark = 3;
				b++;
				break;

			case 3:
				if (!--b) mark = 1;
				r++;
				break;

			default:
				r = 255; g = 0;
				b = 0; mark = 1;
		}

		DeleteObject(hpen);
		hpen = CreatePen(PS_SOLID,1,RGB(r,g,b));
		SelectObject(hdc,hpen);

		MoveToEx(hdc,x1,y1,NULL);
		LineTo(hdc,x2,y2);

		/*  fps  30 */
		timeout = 33 - (GetTickCount() - dwStart);
		if (timeout > 2)
			Sleep(timeout-2);
		while ((GetTickCount() - dwStart) < 33);
	}

	ReleaseDC(ghWndMain,hdc);
	DeleteObject(hpen);

	return msg.wParam;
}
